def clean(word):
	""" Removes punctuation and other non-alphabetical characters from the word.
	"""
	word = word.strip(",;:.?!'`\"-[()]")
	word = word.strip(u"‘“")
	if word.endswith( "'s"): word = word[:-2]
	if word.endswith(u"’s"): word = word[:-2]
	return word.strip()

import en
def count_nouns(txt, filters=[]):
    """ Returns a list of (count, noun)-tuples found in the text.
    Any adjective preceding the noun is picked up as well.
    """
    matches = en.sentence.find(txt.lower(), "(JJ) NN")
    results = {}
    for m in matches:
        noun, tag = m[-1]
        noun = clean(noun)
        if not noun.endswith("ss"):
	        noun = en.noun.singular(noun)
        if len(m) > 1:
	        noun = " ".join([word for word, tag in m])
        if not noun in filters: 
            if not noun in results: 
            	results[noun] = 0
            results[noun] += 1
    results = [(count, noun) for noun, count in results.iteritems()]
    return sorted(results, reverse=True)

def intersect_nouns(txt1, txt2, filters=[], top=None):
	""" Returns a list of (weight, noun)-tuples found in both texts.
	The first text can also be a list prepared with count_nouns().
	"""
	if isinstance(txt1, list):
		nouns = txt1
	else: 
		nouns = count_nouns(txt1, filters)
	if top != None:
		nouns = nouns[:top]
	results = []
	for count, noun in nouns:
		n = txt2.count(noun) * count
		if n > 0:
			results.append((n, noun))
	return sorted(results, reverse=True)	
	

from os.path import basename
graph = ximport("graph")
class lexicon(dict):
	
	def __init__(self, folder=None):
		""" A dictionary of concepts linking to their description.
		"""
		if folder != None:
			for file in files(folder+"*"):
			    concept = basename(file)[:-4]
			    description = open(file).read().decode("utf-8")
			    if not concept in self: 
			    	self[concept] = ""
			    self[concept] += description+" "
		# Concepts are related to each other through nouns in their description.
		# The following nouns are never picked up (to common):
		self.filters = [
			'an', 'and/or', 'and/ori', 'aspect', 'connotation', 'dream', 
			'feeling', 'her', 'her', 'i', 'life', 'metaphor', 'other', 
			'part', 'period', 'person', 'quality', 'sense', 'side', 'situation', 
			'someone', 'someone', 'symbol', 'the', 'the', 'thing', 
			'trait', 'type'
		]
	
	def generate_filters(self, top=20):
		frequency = {}
		for concept, description in self.iteritems():
			for count, noun in count_nouns(description):
				if not noun in frequency:
					frequency[noun] = 0
				frequency[noun] += 1
		frequency = [(count, noun) for noun, count in frequency.iteritems()]
		frequency = sorted(frequency, reverse=True)
		self.filters.extend([noun for count, noun in frequency][:top])
			
	def graph(self, key, top=40, connectivity=8, keywords=7):
		""" Returns a graph for the key and related concepts in the lexicon.
		keywords: the number of nouns to take from the key's description.
		top: the top concepts that also have these keywords in their description.
		connectivity: maximum number of edges for each node in the graph.
		"""
		if key in self:
			nouns = count_nouns(self[key], self.filters+[key])
			nouns = nouns[:keywords]
		else:
			nouns = [(1, key)]
		# Each concept has a score.
		# This score is a list of (count, noun)-tuples found in the concept's description,
		# with each of these nouns also being present in the key's description.
		scores = {}
		for concept, description in lexicon.iteritems():
			if concept != key:
				score = intersect_nouns(nouns, description, self.filters)
				if len(score) > 0:
					scores[concept] = score
		# Scores are sorted based on the total noun count for each score.
		# The result is a list of (concept, [(count, noun), (count, noun), ...])-tuples.
		scores = [(concept, score) for concept, score in scores.iteritems()]
		def total(score):
			return sum([weight for weight, noun in score])
		scores.sort(lambda a, b: total(b[1])-total(a[1]))
		# Relativize all the noun weights between 0.0 and 1.0.
		if len(scores) > 0:
			max = float(scores[0][1][0][0])
			for concept, score in  scores:
				for i in range(len(score)):
					score[i] = (score[i][0]/max, score[i][1])
		# Create the graph from the top scores.
		g = graph.create(distance=1.4)
		g.add_node(key, root=True)
		for concept, score in scores[:top]:
			for count, noun in score:
				# Nouns (i.e. links between concepts) have a maximum
				# number of connections.
				if noun not in g or len(g[noun].links) < connectivity:
					g.add_edge(noun, concept, weight=count)
		if len(nouns) > 0:
			max = float(nouns[0][0])
			nouns = [(count/max, noun) for count, noun in nouns]
		for count, noun in nouns:
			g.add_edge(noun, key, weight=count)
		return g

size(600, 600)
folder = "metaphors/"
lexicon = lexicon(folder)
#lexicon.generate_filters()
#g = lexicon.graph("tuxedo")
#g.solve()
#g.styles.apply()
#g.draw(directed=True, weighted=True, traffic=True)
	
g = None
previous = None
def load(node):
    
    global g
    g = lexicon.graph(str(node))
    g.distance = 1.4
    g.layout.n = 2000
    g.styles.apply()
    g.layout.force = 0.005 # lower if nodes are twitching
    g.events.click = load
    
    global previous
    if previous != None:
		n = g.add_node(previous)
		n.style = "back"
		if len(n.links) == 0: 
			g.add_edge(previous, g.root.id)
    previous = g.root.id	

######################################################################################

load("cat") # <= enter an English word to visualize

######################################################################################

size(600, 600)
speed(30)
def draw():
    g.update()
    g.draw(traffic=True, directed=True)